@@ -1,5 +1,6 @@
 use std::{collections::HashMap, str::FromStr};
 
+mod bookmark_id;
 mod build_xml;
 mod comments;
 mod comments_extended;
@@ -19,12 +20,16 @@ mod header_id;
 mod history_id;
 mod numberings;
 mod paragraph_id;
+mod paragraph_property_change_id;
 mod pic_id;
+mod preset_styles;
 mod rels;
 mod settings;
 mod styles;
 mod taskpanes;
 mod taskpanes_rels;
+mod theme;
+mod toc_key;
 mod web_settings;
 mod webextension;
 mod xml_docx;
@@ -32,8 +37,10 @@ mod xml_docx;
 pub(crate) use build_xml::BuildXML;
 pub(crate) use history_id::HistoryId;
 pub(crate) use paragraph_id::*;
+pub(crate) use paragraph_property_change_id::ParagraphPropertyChangeId;
 pub(crate) use pic_id::*;
 
+pub use bookmark_id::*;
 pub use comments::*;
 pub use comments_extended::*;
 pub use content_types::*;
@@ -55,6 +62,8 @@ pub use settings::*;
 pub use styles::*;
 pub use taskpanes::*;
 pub use taskpanes_rels::*;
+pub use theme::*;
+pub use toc_key::*;
 pub use web_settings::*;
 pub use webextension::*;
 pub use xml_docx::*;
@@ -83,6 +92,8 @@ pub struct Docx {
     pub custom_items: Vec<CustomItem>,
     pub custom_item_props: Vec<CustomItemProperty>,
     pub custom_item_rels: Vec<CustomItemRels>,
+    // reader only
+    pub themes: Vec<Theme>,
 }
 
 impl Default for Docx {
@@ -121,6 +132,7 @@ impl Default for Docx {
             custom_items: vec![],
             custom_item_props: vec![],
             custom_item_rels: vec![],
+            themes: vec![],
         }
     }
 }
@@ -412,11 +424,40 @@ impl Docx {
         self
     }
 
-    pub fn build(&mut self) -> XMLDocx {
+    pub fn build(mut self) -> XMLDocx {
         self.reset();
 
         self.update_comments();
 
+        let tocs: Vec<(usize, TableOfContents)> = self
+            .document
+            .children
+            .iter()
+            .enumerate()
+            .filter_map(|(i, child)| {
+                if let DocumentChild::TableOfContents(toc) = child {
+                    Some((i, toc.clone()))
+                } else {
+                    None
+                }
+            })
+            .collect();
+
+        if !tocs.is_empty() {
+            for i in 1..=9 {
+                self.styles = self
+                    .styles
+                    .add_style(crate::documents::preset_styles::toc(i));
+            }
+        }
+
+        for (i, toc) in tocs {
+            if toc.items.is_empty() && toc.auto {
+                let children = update_document_by_toc(self.document.children, &self.styles, toc, i);
+                self.document.children = children;
+            }
+        }
+
         let (image_ids, images) = self.create_images();
         let web_extensions = self.web_extensions.iter().map(|ext| ext.build()).collect();
         let custom_items = self.custom_items.iter().map(|xml| xml.build()).collect();
@@ -487,6 +528,46 @@ impl Docx {
         crate::reset_para_id();
     }
 
+    fn insert_comment_to_map(
+        &self,
+        comment_map: &mut HashMap<usize, String>,
+        c: &CommentRangeStart,
+    ) {
+        let comment = c.get_comment();
+        let comment_id = comment.id();
+        for child in comment.children {
+            if let CommentChild::Paragraph(child) = child {
+                let para_id = child.id.clone();
+                comment_map.insert(comment_id, para_id.clone());
+            }
+            // TODO: Support table
+        }
+    }
+
+    fn push_comment_and_comment_extended(
+        &self,
+        comments: &mut Vec<Comment>,
+        comments_extended: &mut Vec<CommentExtended>,
+        comment_map: &HashMap<usize, String>,
+        c: &CommentRangeStart,
+    ) {
+        let comment = c.get_comment();
+        for child in comment.children {
+            if let CommentChild::Paragraph(child) = child {
+                let para_id = child.id.clone();
+                comments.push(c.get_comment());
+                let comment_extended = CommentExtended::new(para_id);
+                if let Some(parent_comment_id) = comment.parent_comment_id {
+                    let parent_para_id = comment_map.get(&parent_comment_id).unwrap().clone();
+                    comments_extended.push(comment_extended.parent_paragraph_id(parent_para_id));
+                } else {
+                    comments_extended.push(comment_extended);
+                }
+            }
+            // TODO: Support table
+        }
+    }
+
     // Traverse and clone comments from document and add to comments node.
     fn update_comments(&mut self) {
         let mut comments: Vec<Comment> = vec![];
@@ -498,35 +579,35 @@ impl Docx {
                 DocumentChild::Paragraph(paragraph) => {
                     for child in &paragraph.children {
                         if let ParagraphChild::CommentStart(c) = child {
-                            let comment = c.get_comment();
-                            let comment_id = comment.id();
-                            for child in comment.children {
-                                if let CommentChild::Paragraph(child) = child {
-                                    let para_id = child.id.clone();
-                                    comment_map.insert(comment_id, para_id.clone());
+                            self.insert_comment_to_map(&mut comment_map, c);
+                        }
+                        if let ParagraphChild::Hyperlink(h) = child {
+                            for child in &h.children {
+                                if let ParagraphChild::CommentStart(c) = child {
+                                    self.insert_comment_to_map(&mut comment_map, c);
                                 }
-                                // TODO: Support table
                             }
                         }
                     }
                 }
                 DocumentChild::Table(table) => {
-                    for row in &table.rows {
-                        for cell in &row.cells {
+                    for TableChild::TableRow(row) in &table.rows {
+                        for TableRowChild::TableCell(cell) in &row.cells {
                             for content in &cell.children {
                                 match content {
                                     TableCellContent::Paragraph(paragraph) => {
                                         for child in &paragraph.children {
                                             if let ParagraphChild::CommentStart(c) = child {
-                                                let comment = c.get_comment();
-                                                let comment_id = comment.id();
-                                                for child in comment.children {
-                                                    if let CommentChild::Paragraph(child) = child {
-                                                        let para_id = child.id.clone();
-                                                        comment_map
-                                                            .insert(comment_id, para_id.clone());
+                                                self.insert_comment_to_map(&mut comment_map, c);
+                                            }
+                                            if let ParagraphChild::Hyperlink(h) = child {
+                                                for child in &h.children {
+                                                    if let ParagraphChild::CommentStart(c) = child {
+                                                        self.insert_comment_to_map(
+                                                            &mut comment_map,
+                                                            c,
+                                                        );
                                                     }
-                                                    // TODO: Support table
                                                 }
                                             }
                                         }
@@ -548,59 +629,51 @@ impl Docx {
                 DocumentChild::Paragraph(paragraph) => {
                     for child in &paragraph.children {
                         if let ParagraphChild::CommentStart(c) = child {
-                            let comment = c.get_comment();
-                            for child in comment.children {
-                                if let CommentChild::Paragraph(child) = child {
-                                    let para_id = child.id.clone();
-                                    comments.push(c.get_comment());
-                                    let comment_extended = CommentExtended::new(para_id);
-                                    if let Some(parent_comment_id) = comment.parent_comment_id {
-                                        let parent_para_id =
-                                            comment_map.get(&parent_comment_id).unwrap().clone();
-                                        comments_extended.push(
-                                            comment_extended.parent_paragraph_id(parent_para_id),
-                                        );
-                                    } else {
-                                        comments_extended.push(comment_extended);
-                                    }
+                            self.push_comment_and_comment_extended(
+                                &mut comments,
+                                &mut comments_extended,
+                                &comment_map,
+                                c,
+                            );
+                        }
+                        if let ParagraphChild::Hyperlink(h) = child {
+                            for child in &h.children {
+                                if let ParagraphChild::CommentStart(c) = child {
+                                    self.push_comment_and_comment_extended(
+                                        &mut comments,
+                                        &mut comments_extended,
+                                        &comment_map,
+                                        c,
+                                    );
                                 }
-                                // TODO: Support table
                             }
                         }
                     }
                 }
                 DocumentChild::Table(table) => {
-                    for row in &table.rows {
-                        for cell in &row.cells {
+                    for TableChild::TableRow(row) in &table.rows {
+                        for TableRowChild::TableCell(cell) in &row.cells {
                             for content in &cell.children {
                                 match content {
                                     TableCellContent::Paragraph(paragraph) => {
                                         for child in &paragraph.children {
                                             if let ParagraphChild::CommentStart(c) = child {
-                                                let comment = c.get_comment();
-                                                for child in comment.children {
-                                                    if let CommentChild::Paragraph(child) = child {
-                                                        let para_id = child.id.clone();
-                                                        comments.push(c.get_comment());
-                                                        let comment_extended =
-                                                            CommentExtended::new(para_id);
-                                                        if let Some(parent_comment_id) =
-                                                            comment.parent_comment_id
-                                                        {
-                                                            let parent_para_id = comment_map
-                                                                .get(&parent_comment_id)
-                                                                .unwrap()
-                                                                .clone();
-                                                            comments_extended.push(
-                                                                comment_extended
-                                                                    .parent_paragraph_id(
-                                                                        parent_para_id,
-                                                                    ),
-                                                            );
-                                                        } else {
-                                                            comments_extended
-                                                                .push(comment_extended);
-                                                        }
+                                                self.push_comment_and_comment_extended(
+                                                    &mut comments,
+                                                    &mut comments_extended,
+                                                    &comment_map,
+                                                    c,
+                                                );
+                                            }
+                                            if let ParagraphChild::Hyperlink(h) = child {
+                                                for child in &h.children {
+                                                    if let ParagraphChild::CommentStart(c) = child {
+                                                        self.push_comment_and_comment_extended(
+                                                            &mut comments,
+                                                            &mut comments_extended,
+                                                            &comment_map,
+                                                            c,
+                                                        );
                                                     }
                                                 }
                                             }
@@ -671,8 +744,8 @@ impl Docx {
                     }
                 }
                 DocumentChild::Table(table) => {
-                    for row in &mut table.rows {
-                        for cell in &mut row.cells {
+                    for TableChild::TableRow(row) in &mut table.rows {
+                        for TableRowChild::TableCell(cell) in &mut row.cells {
                             for content in &mut cell.children {
                                 match content {
                                     TableCellContent::Paragraph(paragraph) => {
@@ -760,8 +833,8 @@ impl Docx {
                     }
                 }
                 DocumentChild::Table(table) => {
-                    for row in &mut table.rows {
-                        for cell in &mut row.cells {
+                    for TableChild::TableRow(row) in &mut table.rows {
+                        for TableRowChild::TableCell(cell) in &mut row.cells {
                             for content in &mut cell.children {
                                 match content {
                                     TableCellContent::Paragraph(paragraph) => {
@@ -795,3 +868,101 @@ impl Docx {
         (image_ids, images)
     }
 }
+
+fn update_document_by_toc(
+    document_children: Vec<DocumentChild>,
+    styles: &Styles,
+    toc: TableOfContents,
+    toc_index: usize,
+) -> Vec<DocumentChild> {
+    let heading_map = styles.create_heading_style_map();
+    let mut items = vec![];
+    let mut children = vec![];
+    let style_map: std::collections::HashMap<String, usize> = toc
+        .instr
+        .styles_with_levels
+        .iter()
+        .map(|sl| sl.0.clone())
+        .collect();
+
+    if toc.instr.heading_styles_range.is_none() && !toc.instr.styles_with_levels.is_empty() {
+        // INFO: if \t option set without heading styles ranges, Microsoft word does not show ToC items...
+        return document_children;
+    }
+
+    let (min, max) = toc.instr.heading_styles_range.unwrap_or((0, 9));
+
+    for child in document_children.into_iter() {
+        match child {
+            DocumentChild::Paragraph(mut paragraph) => {
+                if let Some(heading_level) = paragraph
+                    .property
+                    .style
+                    .as_ref()
+                    .map(|p| p.val.to_string())
+                    .and_then(|sid| heading_map.get(&sid))
+                {
+                    if min <= *heading_level && max >= *heading_level {
+                        let toc_key = TocKey::generate();
+                        items.push(
+                            TableOfContentsItem::new()
+                                .text(paragraph.raw_text())
+                                .toc_key(&toc_key)
+                                .level(*heading_level),
+                        );
+                        paragraph = paragraph.wrap_by_bookmark(generate_bookmark_id(), &toc_key);
+                    }
+
+                    if let Some((_min, _max)) = toc.instr.tc_field_level_range {
+                        // TODO: check tc field
+                    }
+                }
+
+                // Support \t option. Collect toc items if style id matched.
+                if let Some(level) = paragraph
+                    .property
+                    .style
+                    .as_ref()
+                    .and_then(|s| style_map.get(&s.val))
+                {
+                    if min <= *level && max >= *level {
+                        let toc_key = TocKey::generate();
+                        items.push(
+                            TableOfContentsItem::new()
+                                .text(paragraph.raw_text())
+                                .toc_key(&toc_key)
+                                .level(*level),
+                        );
+                        paragraph = paragraph.wrap_by_bookmark(generate_bookmark_id(), &toc_key);
+                    }
+                }
+
+                children.push(DocumentChild::Paragraph(paragraph));
+            }
+            DocumentChild::Table(ref _table) => {
+                // TODO:
+                // for row in &table.rows {
+                //     for cell in &row.cells {
+                //         for content in &cell.children {
+                //             match content {
+                //                 TableCellContent::Paragraph(paragraph) => {}
+                //                 TableCellContent::Table(_) => {
+                //                     // TODO: Support table in table
+                //                 }
+                //             }
+                //         }
+                //     }
+                // }
+                children.push(child);
+            }
+            _ => {
+                children.push(child);
+            }
+        }
+    }
+
+    let mut toc = toc;
+    toc.items = items;
+    children[toc_index] = DocumentChild::TableOfContents(toc);
+    children
+}
