@@ -29,6 +29,7 @@ pub enum XMLElement {
     Highlight,
     VertAlign,
     Bold,
+    RunStyle,
     BoldCs,
     Break,
     Tab,
@@ -235,6 +236,7 @@ impl FromStr for XMLElement {
             "pStyle" => Ok(XMLElement::ParagraphStyle),
             "pPrChange" => Ok(XMLElement::ParagraphPropertyChange),
             "highlight" => Ok(XMLElement::Highlight),
+            "rStyle" => Ok(XMLElement::RunStyle),
             "b" => Ok(XMLElement::Bold),
             "bCs" => Ok(XMLElement::BoldCs),
             "i" => Ok(XMLElement::Italic),
